#include "profile.h"
#include <opencv2/imgcodecs.hpp>
#include <QStandardPaths>
#include <QFile>
#include <QDir>

void CameraSetup::store() const
{
	cv::imwrite((camerasLocation() + QString::number(id) + ".darkmap.png").toStdString(), darkmap);
}

void CameraSetup::load(size_t cameraId)
{
	id = cameraId;
	darkmap = cv::imread((camerasLocation() + QString::number(id) + ".darkmap.png").toStdString());
}

QString CameraSetup::camerasLocation()
{
	return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/cameras/";
}

void LightingSetup::store(QSettings& settings) const
{
	settings.setValue(GROUP + QString("/brightness"), brightness);
	settings.setValue(GROUP + QString("/mask"), mask);
}

void LightingSetup::load(const QSettings& settings)
{
	brightness = settings.value(GROUP + QString("/brightness"), 0.25).toDouble();
	mask = settings.value(GROUP + QString("/maks"), 0).toUInt();
}

void Profile::store(QSettings& settings) const
{
	lighting.store(settings);
	settings.setValue(GROUP + QString("/id"), static_cast<unsigned long long>(id));
	settings.setValue(GROUP + QString("/exposureTime"), exposureTime);
	settings.setValue(GROUP + QString("/name"), name_);

	cv::imwrite((profileLocation() + QString::number(id) + ".lightmap.png").toStdString(), lightmap);

	settings.beginWriteArray(GROUP + QString("/cameras"), cameras.size());
	for(size_t i = 0; i < cameras.size(); ++i)
	{
		const CameraSetup& camera = cameras[i];
		settings.setArrayIndex(i);
		camera.store();
		settings.setValue("id", static_cast<unsigned long long>(camera.id));
	}
	settings.endArray();
}

void Profile::load(QSettings &settings)
{
	lighting.load(settings);
	id = settings.value(GROUP + QString("/id")).toULongLong();
	exposureTime = settings.value(GROUP + QString("/exposureTime"), 1.0/60.0).toDouble();
	name_ = settings.value(GROUP + QString("/name"), "NULL").toString();

	lightmap = cv::imread((profileLocation() + QString::number(id) + ".lightmap.png").toStdString());

	int size = settings.beginReadArray(GROUP + QString("/cameras"));
	for(int i = 0; i < size; ++i)
	{
		settings.setArrayIndex(i);
		cameras.push_back(CameraSetup());
		cameras.back().load(settings.value("id", 0).toULongLong());
	}
	settings.endArray();
}

void Profile::load(const QString& name)
{
	QSettings settings(profileLocation() + name + ".profile.ini");
	name_=name;
	load(settings);
}

void Profile::store(const QString& name)
{
	QSettings settings(profileLocation() + name + ".profile.ini");
	name_=name;
	store(settings);
}

void Profile::load()
{
	load(name_);
}

void Profile::store()
{
	store(name_);
}

void Profile::deleteProfile()
{
	QFile::remove(profileLocation() + name_ + ".profile.ini");
}

QList<QString> Profile::avaiableProfiles()
{
	QDir dir(profileLocation());
	dir.setFilter(QDir::Files);
	QStringList filters;
	filters<<"*.ini";
	dir.setNameFilters(filters);
	QList<QString> ret;
	QStringList list = dir.entryList();

	for(QString string : list)
		ret.push_back(string.split(".")[0]);
	return  ret;
}

bool Profile::camerasSufficant(const std::vector<cam::Camera::Description>& desc) const
{
	bool invalid = false;
	for(auto& cameraProfile : cameras)
	{
		bool found = false;
		for(auto& camera : desc)
		{
			if(camera.getHash() == cameraProfile.id)
			{
				found = true;
				break;
			}
		}

		if(!found)
		{
			invalid = true;
			break;
		}
	}
	return  !invalid;
}


QString Profile::profileLocation()
{
	return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/profiles/";
}

Profile::Profile(const QString& name): name_(name)
{
	id = QRandomGenerator::system()->generate64();
}
